@@ -7,9 +7,9 @@ import matplotlib.pyplot as plt
 import types
 
 
-# generic class only implement algo. didn't implement detail func about M0, UT, F1 or F2
+# generic class only implement algo. didn't implement detail func about M0, UT, etc.
 class MfgBase(object):
-    # it's a base class which implements the policy iteration and fixed point iteration
+    # it's a base class which implements the policy iteration
     def __init__(
         self, Q_guess=None, nv=0.05, x1=0, x2=1, Nx=200, Nt=200, T=1.0, bc="periodic"
     ):
@@ -160,7 +160,7 @@ class MfgBase(object):
         # like UT = np.sum(self.xs*mT)*self.dx*self.xs
         return self.UT
 
-    # hamilton
+    # default hamilton
     # H(Du,m) = 0.5*|Du|^2/F1(m) - F2(m)
     def F1_func(self, m):
         raise NotImplementedError(self.__class__.__name__ + "didn't implement F1_func")
@@ -168,20 +168,25 @@ class MfgBase(object):
     def F2_func(self, m):
         raise NotImplementedError(self.__class__.__name__ + "didn't implement F2_func")
 
-    def hjb_rhs(self, un, q, mn):
-        """
-        rhs of HJB equation
-        """
-        ql, qr = q  # unpack
-        return un + self.dt * (
-            0.5 * self.F1_func(mn) * (ql**2 + qr**2) + self.F2_func(mn)
-        )
+    def optimal_control(self, m, du):
+        return du / self.F1_func(m)
 
-    def hjb_rhs_fixp(self, un, du2, mn):
+    def running_cost(self, m, q):
+        ql, qr = q  # unpack
+        q2 = ql**2 + qr**2
+        return 0.5 * self.F1_func(m) * q2 + self.F2_func(m)
+
+    def hjb_rhs(self, u, q, m):
         """
         rhs of HJB equation
         """
-        rhs = un - self.dt * (0.5 * du2 / self.F1_func(mn) - self.F2_func(mn))
+        return u + self.dt * self.running_cost(m, q)
+
+    def hjb_rhs_fixp(self, u, du2, m):
+        """
+        rhs of HJB equation
+        """
+        rhs = u - self.dt * (0.5 * du2 / self.F1_func(m) - self.F2_func(m))
         return rhs
 
     def logging_solution(self, M, U):
@@ -271,8 +276,10 @@ class MfgBase(object):
         for ti in range(self.Nt):
             mn = M[ti, :]
             un = U[ti, :]
-            self.QL[ti, :] = np.maximum((self.DL @ un) / self.F1_func(mn), 0.0)
-            self.QR[ti, :] = np.minimum((self.DR @ un) / self.F1_func(mn), 0.0)
+            dul = self.DL @ un
+            dur = self.DR @ un
+            self.QL[ti, :] = np.maximum(self.optimal_control(mn, dul), 0.0)
+            self.QR[ti, :] = np.minimum(self.optimal_control(mn, dur), 0.0)
         # compute the diff bettwen Q and Q_old
         diff_QL = np.max(np.abs(self.QL - self.QL_old))
         diff_QR = np.max(np.abs(self.QR - self.QR_old))
@@ -433,9 +440,11 @@ class MfgBase(object):
         for ti in range(self.Nt):
             m = M[ti, :]
             un = U[ti, :]
+            dul = self.DL @ un
+            dur = self.DR @ un
             # q here is only a temp variable to compute residual
-            ql = np.maximum((self.DL @ un) / self.F1_func(m), 0.0)
-            qr = np.minimum((self.DR @ un) / self.F1_func(m), 0.0)
+            ql = np.maximum(self.optimal_control(m, dul), 0.0)
+            qr = np.minimum(self.optimal_control(m, dur), 0.0)
             q = [ql, qr]
             lhsM = self.linearOP_FP(q) / self.dt
             rhs = mn / self.dt
@@ -451,9 +460,11 @@ class MfgBase(object):
         for ti in range(self.Nt - 1, -1, -1):
             mn = M[ti, :]
             u = U[ti, :]
+            dul = self.DL @ u
+            dur = self.DR @ u
             # q here is only a temp variable to compute residual
-            ql = np.maximum((self.DL @ u) / self.F1_func(mn), 0.0)
-            qr = np.minimum((self.DR @ u) / self.F1_func(mn), 0.0)
+            ql = np.maximum(self.optimal_control(mn, dul), 0.0)
+            qr = np.minimum(self.optimal_control(mn, dur), 0.0)
             q = [ql, qr]
             lhsM = self.linearOP_HJB(q) / self.dt
             rhs = self.hjb_rhs(un, q, mn) / self.dt
@@ -469,8 +480,11 @@ class MfgBase(object):
         for ti in range(self.Nt):
             m = self.M[ti, :]
             u = self.U[ti, :]
-            ql = np.maximum((self.DL @ u) / self.F1_func(m), 0.0)
-            qr = np.minimum((self.DR @ u) / self.F1_func(m), 0.0)
+            dul = self.DL @ u
+            dur = self.DR @ u
+            # q here is only a temp variable to compute residual
+            ql = np.maximum(self.optimal_control(m, dul), 0.0)
+            qr = np.minimum(self.optimal_control(m, dur), 0.0)
             q2 = ql**2 + qr**2
             cost += np.sum(
                 self.dt * self.dx * m * (0.5 * self.F1_func(m) * q2 + self.F2_func(m))
@@ -656,19 +670,25 @@ class MfgBase2D(object):
     def F2_func(self, m):
         raise NotImplementedError(self.__class__.__name__ + "didn't implement F2_func")
 
-    def hjb_rhs(self, un, q, mn):
-        ql1, qr1, ql2, qr2 = q
-        rhs = un + self.dt * (
-            0.5 * self.F1_func(mn) * (ql1**2 + qr1**2 + ql2**2 + qr2**2)
-            + self.F2_func(mn)
-        )
-        return rhs
+    def optimal_control(self, m, du):
+        return du / self.F1_func(m)
 
-    def hjb_rhs_fixp(self, un, du2, mn):
+    def running_cost(self, m, q):
+        ql1, qr1, ql2, qr2 = q  # unpack
+        q2 = ql1**2 + qr1**2 + ql2**2 + qr2**2
+        return 0.5 * self.F1_func(m) * q2 + self.F2_func(m)
+
+    def hjb_rhs(self, u, q, m):
+        """
+        rhs of HJB equation
+        """
+        return u + self.dt * self.running_cost(m, q)
+
+    def hjb_rhs_fixp(self, u, du2, m):
         """
         rhs of HJB equation in fixpoint method
         """
-        rhs = un - self.dt * (0.5 * du2 / self.F1_func(mn) - self.F2_func(mn))
+        rhs = u - self.dt * (0.5 * du2 / self.F1_func(m) - self.F2_func(m))
         return rhs
 
     def logging_solution(self, M, U):
@@ -768,10 +788,14 @@ class MfgBase2D(object):
         for ti in range(self.Nt):
             mn = M[ti, :]
             un = U[ti, :]
-            self.QL1[ti, :] = np.maximum((self.DL1 @ un) / self.F1_func(mn), 0.0)
-            self.QR1[ti, :] = np.minimum((self.DR1 @ un) / self.F1_func(mn), 0.0)
-            self.QL2[ti, :] = np.maximum((self.DL2 @ un) / self.F1_func(mn), 0.0)
-            self.QR2[ti, :] = np.minimum((self.DR2 @ un) / self.F1_func(mn), 0.0)
+            dul1 = self.DL1 @ un
+            dur1 = self.DR1 @ un
+            dul2 = self.DL2 @ un
+            dur2 = self.DR2 @ un
+            self.QL1[ti, :] = np.maximum(self.optimal_control(mn, dul1), 0.0)
+            self.QR1[ti, :] = np.minimum(self.optimal_control(mn, dur1), 0.0)
+            self.QL2[ti, :] = np.maximum(self.optimal_control(mn, dul2), 0.0)
+            self.QR2[ti, :] = np.minimum(self.optimal_control(mn, dur2), 0.0)
         self.QL1_old = np.copy(self.QL1)
         self.QR1_old = np.copy(self.QR1)
         self.QL2_old = np.copy(self.QL2)
@@ -945,11 +969,15 @@ class MfgBase2D(object):
         for ti in range(self.Nt):
             m = M[ti, :]
             un = U[ti, :]
+            dul1 = self.DL1 @ un
+            dur1 = self.DR1 @ un
+            dul2 = self.DL2 @ un
+            dur2 = self.DR2 @ un
             # q here is only a temp variable to compute residual
-            ql1 = np.maximum((self.DL1 @ un) / self.F1_func(m), 0.0)
-            qr1 = np.minimum((self.DR1 @ un) / self.F1_func(m), 0.0)
-            ql2 = np.maximum((self.DL2 @ un) / self.F1_func(m), 0.0)
-            qr2 = np.minimum((self.DR2 @ un) / self.F1_func(m), 0.0)
+            ql1 = np.maximum(self.optimal_control(m, dul1), 0.0)
+            qr1 = np.minimum(self.optimal_control(m, dur1), 0.0)
+            ql2 = np.maximum(self.optimal_control(m, dul2), 0.0)
+            qr2 = np.minimum(self.optimal_control(m, dur2), 0.0)
             q = [ql1, qr1, ql2, qr2]
             lhsM = self.linearOP_FP(q) / self.dt
             rhs = mn / self.dt
@@ -965,10 +993,15 @@ class MfgBase2D(object):
         for ti in range(self.Nt - 1, -1, -1):
             mn = M[ti, :]
             u = U[ti, :]
-            ql1 = np.maximum((self.DL1 @ u) / self.F1_func(mn), 0.0)
-            qr1 = np.minimum((self.DR1 @ u) / self.F1_func(mn), 0.0)
-            ql2 = np.maximum((self.DL2 @ u) / self.F1_func(mn), 0.0)
-            qr2 = np.minimum((self.DR2 @ u) / self.F1_func(mn), 0.0)
+            dul1 = self.DL1 @ u
+            dur1 = self.DR1 @ u
+            dul2 = self.DL2 @ u
+            dur2 = self.DR2 @ u
+            # q here is only a temp variable to compute residual
+            ql1 = np.maximum(self.optimal_control(mn, dul1), 0.0)
+            qr1 = np.minimum(self.optimal_control(mn, dur1), 0.0)
+            ql2 = np.maximum(self.optimal_control(mn, dul2), 0.0)
+            qr2 = np.minimum(self.optimal_control(mn, dur2), 0.0)
             q = [ql1, qr1, ql2, qr2]
             lhsM = self.linearOP_HJB(q) / self.dt
             rhs = self.hjb_rhs(un, q, mn) / self.dt
@@ -979,15 +1012,20 @@ class MfgBase2D(object):
 
     def compute_cost(self):
         cost = 0.0
-        mT = M[-1, :]
+        mT = self.M[-1, :]
         uT = self.get_UT(mT)
         for ti in range(self.Nt):
             m = self.M[ti, :]
             u = self.U[ti, :]
-            ql1 = np.maximum((self.DL1 @ u) / self.F1_func(m), 0.0)
-            qr1 = np.minimum((self.DR1 @ u) / self.F1_func(m), 0.0)
-            ql2 = np.maximum((self.DL2 @ u) / self.F1_func(m), 0.0)
-            qr2 = np.minimum((self.DR2 @ u) / self.F1_func(m), 0.0)
+            dul1 = self.DL1 @ u
+            dur1 = self.DR1 @ u
+            dul2 = self.DL2 @ u
+            dur2 = self.DR2 @ u
+            # q here is only a temp variable to compute residual
+            ql1 = np.maximum(self.optimal_control(m, dul1), 0.0)
+            qr1 = np.minimum(self.optimal_control(m, dur1), 0.0)
+            ql2 = np.maximum(self.optimal_control(m, dul2), 0.0)
+            qr2 = np.minimum(self.optimal_control(m, dur2), 0.0)
             q2 = ql1**2 + qr1**2 + ql2**2 + qr2**2
             cost += np.sum(
                 self.dt
